const parser = require("@babel/parser");
const generator = require("@babel/generator").default;
const template = require("@babel/template").default;
const traverse = require("@babel/traverse").default;
const codeFrame = require("@babel/code-frame");

class CodeEditor {
  constructor(code) {
    if (!code) {
      throw new Error("code不能为空");
    }
    this.ast = this._parse(code);
  }
  _parse(code) {
    return parser.parse(code, { sourceType: "module" });
  }
  /**
   * 更新变量值
   * @param {String} key 变量名
   * @param {String} value 变量值
   */
  updateVariable(key, value) {
    traverse(this.ast, {
      VariableDeclarator(path) {
        if (path.node.id.name === key) {
          path.node.init = template.ast`${value}`;
        }
      }
    });
  }
  /**
   * 得到import的说明符
   * @param {String} source 源
   * @return {Array}
   * @example import {a,b,c} from 'file.js' => [a,b,c]
   */
  getImportSpecifier(source) {
    let val = [];
    let _self = this;
    traverse(this.ast, {
      ImportDeclaration(path) {
        if (path.node.source.value === source) {
          val = path.node.specifiers.map(item => {
            return _self.generate(item);
          });
        }
      }
    });
    return val;
  }

  replaceImport(source, code) {
    traverse(this.ast, {
      ImportDeclaration(path) {
        if (path.node.source.value === source) {
          path.replaceWith(template.ast`${code}`);
        }
      }
    });
  }

  /**
   * 添加代码
   * @param {String} code 插入的代码
   * @param {String} location 位置：after、before
   */
  addCode(code, location = "after") {
    traverse(this.ast, {
      Program(path) {
        if (location === "after") {
          path.node.body.push(template.ast`${code}`);
        } else if (location === "before") {
          path.node.body.unshift(template.ast`${code}`);
        }
      }
    });
  }

  generate(ast) {
    const code = generator(ast || this.ast).code;
    try {
      this._parse(code);
    } catch (error) {
      const result = codeFrame.codeFrameColumns(
        code,
        { start: error.loc },
        {
          highlightCode: true,
          message: "生成的代码解析出错:" + error.message,
          linesAbove: 999,
          linesBelow: 999
        }
      );
      console.error(`\n${result}`);
      process.exit(0)
    }

    return code;
  }
}

module.exports = CodeEditor;