#!/usr/bin/env node

/**
 * 1.读取项目文件，获取项目地址,判断是否小程序 ✔︎
 * 2.打开IDE ✔︎
 * 3.切换环境
 * 3.切换页面(无法切换页面)
 * 4.上传代码,自动加上备注
 * 5.注册开发工具 ✔︎
 * 6.调试选项
 * 7.添加page或components
 */

const spawn = require("cross-spawn");

const path = require("path");
const fs = require("fs-extra");
const os = require("os");
const CodeEditor = require("./codeEditor");
const prompts = require("./prompts");

const db_path = path.join(os.homedir(), ".minitool.json");

const project_path = process.cwd();

const project_build_config_path = path.join(project_path, "build-config.js");
const project_app_path = path.join(project_path, "app.json");

let cli = "";

function getDB() {
  if (!fs.existsSync(db_path)) {
    setDB({});
  }
  return fs.readJSONSync(db_path);
}

function setDB(obj) {
  fs.writeJSONSync(db_path, obj);
}

function isWindows() {
  if (typeof process !== "undefined") {
    return (
      process.platform === "win32" ||
      process.env.OSTYPE === "cygwin" ||
      process.env.OSTYPE === "msys"
    );
  }
}

function cliPath() {
  const path = isWindows()
    ? "C:/Program Files (x86)/Tencent/微信web开发者工具/cli.bat"
    : "/Applications/wechatwebdevtools.app/Contents/MacOS/cli";
  if (fs.existsSync(path)) {
    return path;
  } else if (hasCustomCliPath()) {
    return getCustomCliPath();
  } else {
    console.error("未发现cli,请先安装微信IDE或者注册cli");
    process.exit(0);
  }
}

function hasCustomCliPath() {
  return fs.existsSync(getCustomCliPath());
}

function getCustomCliPath() {
  return getDB().cli;
}

function initCustomCli(path) {
  if (fs.pathExistsSync(path)) {
    setDB({
      cli: path
    });
  } else {
    console.log(`输入的地址不存在`);
  }
}

function isMiniProgram() {
  const mini_config_path = path.join(project_path, "project.config.json");
  if (fs.existsSync(mini_config_path)) {
    const mini_config = require(mini_config_path);
    return !!mini_config.compileType;
  } else {
    return false;
  }
}

async function openIDE() {
  await spawn(cli, ["-o", project_path]);
}

async function login() {
  await spawn(
    cli,
    ["--login", "--login-result-output", `${project_path}/result.json`],
    {
      stdio: "inherit"
    }
  );
}
async function preview(page) {
  if (page) {
    await spawn(
      cli,
      ["-p", project_path, "--compile-condition", `{"pathName":"${page}"}`],
      {
        stdio: "inherit"
      }
    );
  } else {
    await spawn(cli, ["-p", project_path], {
      stdio: "inherit"
    });
  }
}

async function autoPreview() {
  await spawn(cli, ["--auto-preview", project_path], {
    stdio: "inherit"
  });
}

async function upload(v, des) {
  await spawn(cli, ["-u", `${v}@${project_path}`, "--upload-desc", des], {
    stdio: "inherit"
  });
}

let fileSource = "";

function setBuildEnv(val) {
  const codeEditor = new CodeEditor(fileSource);
  codeEditor.updateVariable("build_env", val);
  fileSource = codeEditor.generate();
}

function setBuildVersion(val) {
  const codeEditor = new CodeEditor(fileSource);
  codeEditor.updateVariable("mini_tool_version", val);
  fileSource = codeEditor.generate();
}

function readFile() {
  fileSource = fs.readFileSync(project_build_config_path, {
    encoding: "utf-8"
  });
}
function writeFile() {
  fs.writeFileSync(project_build_config_path, fileSource);
}

async function run() {
  let reuslt = await prompts.runPrompts();
  if (reuslt === "initCli") {
    reuslt = await prompts.runInitCli();
    initCustomCli(reuslt);
    process.exit(0);
  }
  cli = cliPath();
  if (!isMiniProgram()) {
    console.log("当前目录不是小程序项目，无法执行。");
    process.exit(0);
  }
  let buildConfig = require(project_build_config_path);
  let pageList = require(project_app_path).pages;
  switch (reuslt) {
    case "start":
      reuslt = await prompts.runToggleEnv(buildConfig.envList);
      readFile();
      setBuildEnv(`'${reuslt}'`);
      writeFile();
      openIDE();
      break;
    case "togger":
      reuslt = await prompts.runToggleEnv(buildConfig.envList);
      readFile();
      setBuildEnv(`'${reuslt}'`);
      writeFile();
      break;
    case "upload":
      let env = await prompts.runToggleEnv(buildConfig.envList);
      readFile();
      setBuildEnv(`'${env}'`);
      reuslt = await prompts.runSetDescriptionAndVersion();
      setBuildVersion(`'${reuslt.v}'`);
      writeFile();
      await upload(reuslt.v, reuslt.des + `,当前环境：${env}`);
      break;
    case "preview":
      result = await prompts.runTogglePage(pageList);
      await preview(result);
      break;
    case "autoPreview":
      await autoPreview();
      break;
    case "info":
      console.log(`当前版本：${buildConfig.version}`);
      console.log(`当前环境：${buildConfig.env}`);
      console.log(
        `当前测试账号：${buildConfig.account.user}  ${buildConfig.account.pwd}`
      );
      break;
  }
}

run();
